@@ -31,19 +31,10 @@ pub mod time;
 mod type_matcher;
 mod user;
 
-use ::regex::Regex;
-use chrono::{DateTime, Datelike, NaiveDateTime, Utc};
-use fs::FileSystemMatcher;
-use ls::Ls;
-use std::fs::{File, Metadata};
-use std::path::Path;
-use std::time::SystemTime;
-use std::{error::Error, str::FromStr};
-
 use self::access::AccessMatcher;
 use self::delete::DeleteMatcher;
 use self::empty::EmptyMatcher;
-use self::exec::SingleExecMatcher;
+use self::exec::{MultiExecMatcher, SingleExecMatcher};
 use self::group::{GroupMatcher, NoGroupMatcher};
 use self::lname::LinkNameMatcher;
 use self::logical_matchers::{
@@ -67,6 +58,18 @@ use self::time::{
 };
 use self::type_matcher::{TypeMatcher, XtypeMatcher};
 use self::user::{NoUserMatcher, UserMatcher};
+use ::regex::Regex;
+use chrono::{DateTime, Datelike, NaiveDateTime, Utc};
+use fs::FileSystemMatcher;
+use ls::Ls;
+use std::{
+    error::Error,
+    fs::{File, Metadata},
+    io::Read,
+    path::Path,
+    str::FromStr,
+    time::SystemTime,
+};
 
 use super::{Config, Dependencies};
 
@@ -87,9 +90,9 @@ impl Follow {
     /// Check whether to follow a path of the given depth.
     pub fn follow_at_depth(self, depth: usize) -> bool {
         match self {
-            Follow::Never => false,
-            Follow::Roots => depth == 0,
-            Follow::Always => true,
+            Self::Never => false,
+            Self::Roots => depth == 0,
+            Self::Always => true,
         }
     }
 
@@ -143,7 +146,7 @@ pub struct MatcherIO<'a> {
     deps: &'a dyn Dependencies,
 }
 
-impl<'a> MatcherIO<'a> {
+impl MatcherIO<'_> {
     pub fn new(deps: &dyn Dependencies) -> MatcherIO<'_> {
         MatcherIO {
             should_skip_dir: false,
@@ -212,13 +215,13 @@ pub trait Matcher: 'static {
         false
     }
 
-    /// Notification that find has finished processing a given directory.
-    fn finished_dir(&self, _finished_directory: &Path) {}
+    /// Notification that find is leaving a given directory.
+    fn finished_dir(&self, _finished_directory: &Path, _matcher_io: &mut MatcherIO) {}
 
     /// Notification that find has finished processing all directories -
     /// allowing for any cleanup that isn't suitable for destructors (e.g.
     /// blocking calls, I/O etc.)
-    fn finished(&self) {}
+    fn finished(&self, _matcher_io: &mut MatcherIO) {}
 }
 
 impl Matcher for Box<dyn Matcher> {
@@ -234,15 +237,16 @@ impl Matcher for Box<dyn Matcher> {
         (**self).has_side_effects()
     }
 
-    fn finished_dir(&self, finished_directory: &Path) {
-        (**self).finished_dir(finished_directory);
+    fn finished_dir(&self, finished_directory: &Path, matcher_io: &mut MatcherIO) {
+        (**self).finished_dir(finished_directory, matcher_io);
     }
 
-    fn finished(&self) {
-        (**self).finished();
+    fn finished(&self, matcher_io: &mut MatcherIO) {
+        (**self).finished(matcher_io);
     }
 }
 
+#[derive(Debug, Eq, PartialEq)]
 pub enum ComparableValue {
     MoreThan(u64),
     EqualTo(u64),
@@ -252,18 +256,18 @@ pub enum ComparableValue {
 impl ComparableValue {
     fn matches(&self, value: u64) -> bool {
         match *self {
-            ComparableValue::MoreThan(limit) => value > limit,
-            ComparableValue::EqualTo(limit) => value == limit,
-            ComparableValue::LessThan(limit) => value < limit,
+            Self::MoreThan(limit) => value > limit,
+            Self::EqualTo(limit) => value == limit,
+            Self::LessThan(limit) => value < limit,
         }
     }
 
     /// same as matches, but takes a signed value
     fn imatches(&self, value: i64) -> bool {
         match *self {
-            ComparableValue::MoreThan(limit) => value >= 0 && (value as u64) > limit,
-            ComparableValue::EqualTo(limit) => value >= 0 && (value as u64) == limit,
-            ComparableValue::LessThan(limit) => value < 0 || (value as u64) < limit,
+            Self::MoreThan(limit) => value >= 0 && (value as u64) > limit,
+            Self::EqualTo(limit) => value >= 0 && (value as u64) == limit,
+            Self::LessThan(limit) => value < 0 || (value as u64) < limit,
         }
     }
 }
@@ -308,7 +312,7 @@ fn convert_arg_to_comparable_value(
     option_name: &str,
     value_as_string: &str,
 ) -> Result<ComparableValue, Box<dyn Error>> {
-    let re = Regex::new(r"^([+-]?)(\d+)$")?;
+    let re = Regex::new(r"^([-+]?)[-+]?(\d+)$")?;
     if let Some(groups) = re.captures(value_as_string) {
         if let Ok(val) = groups[2].parse::<u64>() {
             return Ok(match &groups[1] {
@@ -328,7 +332,7 @@ fn convert_arg_to_comparable_value_and_suffix(
     option_name: &str,
     value_as_string: &str,
 ) -> Result<(ComparableValue, String), Box<dyn Error>> {
-    let re = Regex::new(r"([+-]?)(\d+)(.*)$")?;
+    let re = Regex::new(r"([-+]?)[-+]?(\d+)(.*)$")?;
     if let Some(groups) = re.captures(value_as_string) {
         if let Ok(val) = groups[2].parse::<u64>() {
             return Ok((
@@ -453,7 +457,7 @@ fn build_matcher_tree(
                     return Err(From::from(format!("missing argument to {}", args[i])));
                 }
                 i += 1;
-                Some(Printf::new(args[i])?.into_box())
+                Some(Printf::new(args[i], None)?.into_box())
             }
             "-fprint" => {
                 if i >= args.len() - 1 {
@@ -464,6 +468,28 @@ fn build_matcher_tree(
                 let file = get_or_create_file(args[i])?;
                 Some(Printer::new(PrintDelimiter::Newline, Some(file)).into_box())
             }
+            "-fprintf" => {
+                if i >= args.len() - 2 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+
+                // Action: -fprintf file format
+                // Args + 1: output file path
+                // Args + 2: format string
+                i += 1;
+                let file = get_or_create_file(args[i])?;
+                i += 1;
+                Some(Printf::new(args[i], Some(file))?.into_box())
+            }
+            "-fprint0" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+
+                let file = get_or_create_file(args[i])?;
+                Some(Printer::new(PrintDelimiter::Null, Some(file)).into_box())
+            }
             "-ls" => Some(Ls::new(None).into_box()),
             "-fls" => {
                 if i >= args.len() - 1 {
@@ -598,29 +624,49 @@ fn build_matcher_tree(
             "-empty" => Some(EmptyMatcher::new().into_box()),
             "-exec" | "-execdir" => {
                 let mut arg_index = i + 1;
-                while arg_index < args.len() && args[arg_index] != ";" {
-                    if args[arg_index - 1] == "{}" && args[arg_index] == "+" {
-                        // MultiExecMatcher isn't written yet
-                        return Err(From::from(format!(
-                            "{} [args...] + isn't supported yet. \
-                             Only {} [args...] ;",
-                            args[i], args[i]
-                        )));
-                    }
+                while arg_index < args.len()
+                    && args[arg_index] != ";"
+                    && (args[arg_index - 1] != "{}" || args[arg_index] != "+")
+                {
                     arg_index += 1;
                 }
-                if arg_index < i + 2 || arg_index == args.len() {
+                let required_arg = if arg_index < args.len() && args[arg_index] == "+" {
+                    3
+                } else {
+                    2
+                };
+                if arg_index < i + required_arg || arg_index == args.len() {
                     // at the minimum we need the executable and the ';'
+                    // or the executable and the '{} +'
                     return Err(From::from(format!("missing argument to {}", args[i])));
                 }
                 let expression = args[i];
                 let executable = args[i + 1];
                 let exec_args = &args[i + 2..arg_index];
                 i = arg_index;
-                Some(
-                    SingleExecMatcher::new(executable, exec_args, expression == "-execdir")?
-                        .into_box(),
-                )
+                match args[arg_index] {
+                    ";" => Some(
+                        SingleExecMatcher::new(executable, exec_args, expression == "-execdir")?
+                            .into_box(),
+                    ),
+                    "+" => {
+                        if exec_args.iter().filter(|x| matches!(**x, "{}")).count() == 1 {
+                            Some(
+                                MultiExecMatcher::new(
+                                    executable,
+                                    &exec_args[0..exec_args.len() - 1],
+                                    expression == "-execdir",
+                                )?
+                                .into_box(),
+                            )
+                        } else {
+                            return Err(From::from(
+                                "Only one instance of {} is supported with -execdir ... +",
+                            ));
+                        }
+                    }
+                    _ => unreachable!("Encountered unexpected value {}", args[arg_index]),
+                }
             }
             #[cfg(unix)]
             "-inum" => {
@@ -672,16 +718,10 @@ fn build_matcher_tree(
                 }
 
                 i += 1;
-                let matcher = UserMatcher::from_user_name(user);
-                match matcher.uid() {
-                    Some(_) => Some(matcher.into_box()),
-                    None => {
-                        return Err(From::from(format!(
-                            "{} is not the name of a known user",
-                            user
-                        )))
-                    }
-                }
+                let matcher = UserMatcher::from_user_name(user)
+                    .or_else(|| Some(UserMatcher::from_uid(user.parse::<u32>().ok()?)))
+                    .ok_or_else(|| format!("{user} is not the name of a known user"))?;
+                Some(matcher.into_box())
             }
             "-nouser" => Some(NoUserMatcher {}.into_box()),
             "-uid" => {
@@ -689,12 +729,9 @@ fn build_matcher_tree(
                     return Err(From::from(format!("missing argument to {}", args[i])));
                 }
                 // check if the argument is a number
-                let uid = args[i + 1].parse::<u32>();
-                if uid.is_err() {
-                    return Err(From::from(format!("{} is not a number", args[i + 1])));
-                }
+                let uid = convert_arg_to_comparable_value(args[i], args[i + 1])?;
                 i += 1;
-                Some(UserMatcher::from_uid(uid.unwrap()).into_box())
+                Some(UserMatcher::from_comparable(uid).into_box())
             }
             "-group" => {
                 if i >= args.len() - 1 {
@@ -710,16 +747,10 @@ fn build_matcher_tree(
                 }
 
                 i += 1;
-                let matcher = GroupMatcher::from_group_name(group);
-                match matcher.gid() {
-                    Some(_) => Some(matcher.into_box()),
-                    None => {
-                        return Err(From::from(format!(
-                            "{} is not the name of an existing group",
-                            group
-                        )))
-                    }
-                }
+                let matcher = GroupMatcher::from_group_name(group)
+                    .or_else(|| Some(GroupMatcher::from_gid(group.parse::<u32>().ok()?)))
+                    .ok_or_else(|| format!("{group} is not the name of an existing group"))?;
+                Some(matcher.into_box())
             }
             "-nogroup" => Some(NoGroupMatcher {}.into_box()),
             "-gid" => {
@@ -727,15 +758,9 @@ fn build_matcher_tree(
                     return Err(From::from(format!("missing argument to {}", args[i])));
                 }
                 // check if the argument is a number
-                let gid = args[i + 1].parse::<u32>();
-                if gid.is_err() {
-                    return Err(From::from(format!(
-                        "find: invalid argument `{}' to `-gid'",
-                        args[i + 1]
-                    )));
-                }
+                let gid = convert_arg_to_comparable_value(args[i], args[i + 1])?;
                 i += 1;
-                Some(GroupMatcher::from_gid(gid.unwrap()).into_box())
+                Some(GroupMatcher::from_comparable(gid).into_box())
             }
             "-executable" => Some(AccessMatcher::Executable.into_box()),
             "-perm" => {
@@ -795,7 +820,9 @@ fn build_matcher_tree(
             }
             ")" => {
                 if !expecting_bracket {
-                    return Err(From::from("you have too many ')'"));
+                    return Err(From::from(
+                        "invalid expression: expected expression before closing parentheses ')'.",
+                    ));
                 }
 
                 let bracket = args[i - 1];
@@ -870,6 +897,14 @@ fn build_matcher_tree(
                 config.version_requested = true;
                 None
             }
+            "-files0-from" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                let _ = config.files0_argument.insert(args[i + 1].to_string());
+                i += 1;
+                Some(TrueMatcher.into_box())
+            }
 
             _ => {
                 match parse_str_to_newer_args(args[i]) {
@@ -885,21 +920,21 @@ fn build_matcher_tree(
                             let time = args[i + 1];
                             let newer_time_type = NewerOptionType::from_str(x_option.as_str());
                             // Convert args to unix timestamps. (expressed in numeric types)
-                            let comparable_time = match parse_date_str_to_timestamps(time) {
-                                Some(timestamp) => timestamp,
-                                None => {
-                                    return Err(From::from(format!(
-                                        "find: I cannot figure out how to interpret ‘{}’ as a date or time",
-                                        args[i + 1]
-                                    )))
-                                }
+                            let Some(comparable_time) = parse_date_str_to_timestamps(time) else {
+                                return Err(From::from(format!(
+                                    "find: I cannot figure out how to interpret ‘{}’ as a date or time",
+                                    args[i + 1]
+                                )));
                             };
                             i += 1;
                             Some(NewerTimeMatcher::new(newer_time_type, comparable_time).into_box())
                         } else {
                             let file_path = args[i + 1];
                             i += 1;
-                            Some(NewerOptionMatcher::new(x_option, y_option, file_path)?.into_box())
+                            Some(
+                                NewerOptionMatcher::new(&x_option, &y_option, file_path)?
+                                    .into_box(),
+                            )
                         }
                     }
                     None => return Err(From::from(format!("Unrecognized flag: '{}'", args[i]))),
@@ -927,9 +962,51 @@ fn build_matcher_tree(
              did not see one.",
         ));
     }
+    if config.files0_argument.is_some() {
+        parse_files0_args(config)?;
+    }
     Ok((i, top_level_matcher.build()))
 }
 
+// https://www.gnu.org/software/findutils/manual/html_node/find_html/Starting-points.html
+// This allows users to take the entry point for find from stdin (eg. pipe) or from a text file.
+// eg. dummy | find -files0-from -
+// eg. find -files0-from rust.txt -name "cargo"
+fn parse_files0_args(config: &mut Config) -> Result<(), Box<dyn Error>> {
+    let mode = config.files0_argument.as_ref().unwrap();
+    let mut buffer = Vec::new();
+    let new_paths = config.new_paths.insert(Vec::new());
+
+    if mode == "-" {
+        std::io::stdin().read_to_end(&mut buffer)?;
+    } else {
+        let mut file =
+            File::open(mode).map_err(|e| format!("cannot open '{}' for reading: {}", mode, e))?;
+        file.read_to_end(&mut buffer)?;
+    }
+
+    let mut buffer_split: Vec<&[u8]> = buffer.split(|&b| b == 0).collect();
+    // if the pipe/file ends with ASCII NULL
+    if buffer_split.last().is_some_and(|s| s.is_empty()) {
+        buffer_split.remove(buffer_split.len() - 1);
+    }
+
+    let mut string_segments: Vec<String> = buffer_split
+        .iter()
+        .filter_map(|s| std::str::from_utf8(s).ok())
+        .map(|s| s.to_string())
+        .collect();
+    // empty starting point checker
+    if string_segments.iter().any(|s| s.is_empty()) {
+        eprintln!("find: invalid zero-length file name");
+        // remove the empty ones so as to avoid file not found error
+        string_segments.retain(|s| !s.is_empty());
+    }
+
+    new_paths.extend(string_segments);
+    Ok(())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1251,7 +1328,9 @@ mod tests {
             &["-type", "f", "(", "-name", "*.txt", ")", ")"],
             &mut config,
         ) {
-            assert!(e.to_string().contains("too many ')'"));
+            assert!(e
+                .to_string()
+                .contains("expected expression before closing parentheses ')'"));
         } else {
             panic!("parsing argument list with too many closing brackets should fail");
         }
@@ -1497,6 +1576,30 @@ mod tests {
         } else {
             panic!("parsing argument list with exec and no executable should fail");
         }
+
+        if let Err(e) = build_top_level_matcher(&["-exec", "+"], &mut config) {
+            assert!(e.to_string().contains("missing argument"));
+        } else {
+            panic!("parsing argument list with exec and no executable should fail");
+        }
+
+        if let Err(e) = build_top_level_matcher(&["-exec", "foo", "+"], &mut config) {
+            assert!(e.to_string().contains("missing argument"));
+        } else {
+            panic!("parsing argument list with exec and no brackets should fail");
+        }
+
+        if let Err(e) = build_top_level_matcher(&["-exec", "{}", "+"], &mut config) {
+            assert!(e.to_string().contains("missing argument"));
+        } else {
+            panic!("parsing argument list with exec and no executable should fail");
+        }
+
+        if let Err(e) = build_top_level_matcher(&["-exec", "foo", "{}", "foo", "+"], &mut config) {
+            assert!(e.to_string().contains("missing argument"));
+        } else {
+            panic!("parsing argument list with exec and + not following {{}} should fail");
+        }
     }
 
     #[test]
@@ -1513,6 +1616,18 @@ mod tests {
             .expect("only {} + should be considered a multi-exec");
     }
 
+    #[test]
+    fn build_top_level_multi_exec_too_many_holders() {
+        let mut config = Config::default();
+        if let Err(e) =
+            build_top_level_matcher(&["-exec", "foo", "{}", "foo", "{}", "+", ";"], &mut config)
+        {
+            assert!(e.to_string().contains("Only one instance of {}"));
+        } else {
+            panic!("parsing argument list with more than one {{}} for + should fail");
+        }
+    }
+
     #[test]
     #[cfg(unix)]
     fn build_top_level_matcher_perm() {
@@ -1567,6 +1682,31 @@ mod tests {
         }
     }
 
+    #[test]
+    fn convert_arg_to_comparable_value_test() {
+        assert_eq!(
+            convert_arg_to_comparable_value("test", "10").unwrap(),
+            ComparableValue::EqualTo(10),
+        );
+        assert_eq!(
+            convert_arg_to_comparable_value("test", "+10").unwrap(),
+            ComparableValue::MoreThan(10),
+        );
+        assert_eq!(
+            convert_arg_to_comparable_value("test", "-10").unwrap(),
+            ComparableValue::LessThan(10),
+        );
+
+        assert_eq!(
+            convert_arg_to_comparable_value("test", "-+10").unwrap(),
+            ComparableValue::LessThan(10),
+        );
+        assert_eq!(
+            convert_arg_to_comparable_value("test", "++10").unwrap(),
+            ComparableValue::MoreThan(10),
+        );
+    }
+
     #[test]
     fn convert_exception_arg_to_comparable_value_test() {
         let exception_args = ["1%2", "1%2%3", "1a2", "1%2a", "abc", "-", "+", "%"];
@@ -1625,7 +1765,7 @@ mod tests {
         let x_options = ["a", "B", "c", "m"];
         let y_options = ["a", "B", "c", "m", "t"];
 
-        for &x in x_options.iter() {
+        for &x in &x_options {
             for &y in &y_options {
                 let eq: (String, String) = (String::from(x), String::from(y));
                 let arg = parse_str_to_newer_args(&format!("-newer{x}{y}").to_string()).unwrap();
