@@ -9,6 +9,8 @@ use std::fs::{self, Metadata};
 use std::io::{stderr, Write};
 use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
+use chrono::{DateTime, Local, Timelike};
+
 #[cfg(unix)]
 use std::os::unix::fs::MetadataExt;
 
@@ -19,10 +21,14 @@ const SECONDS_PER_DAY: i64 = 60 * 60 * 24;
 fn get_time(matcher_io: &mut MatcherIO, today_start: bool) -> SystemTime {
     if today_start {
         // the time at 00:00:00 of today
-        let duration = matcher_io.now().duration_since(UNIX_EPOCH).unwrap();
-        let seconds = duration.as_secs();
-        let midnight_seconds = seconds - (seconds % 86400);
-        UNIX_EPOCH + Duration::from_secs(midnight_seconds)
+        let duration_since_unix_epoch = matcher_io.now().duration_since(UNIX_EPOCH).unwrap();
+        let seconds_since_unix_epoch = duration_since_unix_epoch.as_secs();
+        let utc_time = DateTime::from_timestamp(seconds_since_unix_epoch as i64, 0).unwrap();
+        let local_time = utc_time.with_timezone(&Local);
+        let seconds_since_last_midnight = local_time.num_seconds_from_midnight();
+        let local_midnight_seconds = local_time.timestamp() - seconds_since_last_midnight as i64;
+
+        UNIX_EPOCH + Duration::from_secs(local_midnight_seconds as u64)
     } else {
         matcher_io.now()
     }
@@ -92,19 +98,19 @@ impl NewerOptionType {
     #[allow(clippy::should_implement_trait)]
     pub fn from_str(option: &str) -> Self {
         match option {
-            "a" => NewerOptionType::Accessed,
-            "B" => NewerOptionType::Birthed,
-            "c" => NewerOptionType::Changed,
-            _ => NewerOptionType::Modified,
+            "a" => Self::Accessed,
+            "B" => Self::Birthed,
+            "c" => Self::Changed,
+            _ => Self::Modified,
         }
     }
 
     fn get_file_time(self, metadata: &Metadata) -> std::io::Result<SystemTime> {
         match self {
-            NewerOptionType::Accessed => metadata.accessed(),
-            NewerOptionType::Birthed => metadata.created(),
-            NewerOptionType::Changed => metadata.changed(),
-            NewerOptionType::Modified => metadata.modified(),
+            Self::Accessed => metadata.accessed(),
+            Self::Birthed => metadata.created(),
+            Self::Changed => metadata.changed(),
+            Self::Modified => metadata.modified(),
         }
     }
 }
@@ -118,14 +124,10 @@ pub struct NewerOptionMatcher {
 }
 
 impl NewerOptionMatcher {
-    pub fn new(
-        x_option: String,
-        y_option: String,
-        path_to_file: &str,
-    ) -> Result<Self, Box<dyn Error>> {
+    pub fn new(x_option: &str, y_option: &str, path_to_file: &str) -> Result<Self, Box<dyn Error>> {
         let metadata = fs::metadata(path_to_file)?;
-        let x_option = NewerOptionType::from_str(x_option.as_str());
-        let y_option = NewerOptionType::from_str(y_option.as_str());
+        let x_option = NewerOptionType::from_str(x_option);
+        let y_option = NewerOptionType::from_str(y_option);
         Ok(Self {
             x_option,
             y_option,
@@ -257,9 +259,9 @@ pub enum FileTimeType {
 impl FileTimeType {
     fn get_file_time(self, metadata: &Metadata) -> std::io::Result<SystemTime> {
         match self {
-            FileTimeType::Accessed => metadata.accessed(),
-            FileTimeType::Changed => metadata.changed(),
-            FileTimeType::Modified => metadata.modified(),
+            Self::Accessed => metadata.accessed(),
+            Self::Changed => metadata.changed(),
+            Self::Modified => metadata.modified(),
         }
     }
 }
@@ -396,6 +398,7 @@ impl FileAgeRangeMatcher {
 
 #[cfg(test)]
 mod tests {
+    use chrono::NaiveTime;
     use std::fs;
     use std::fs::{File, OpenOptions};
     use std::io::Read;
@@ -616,6 +619,15 @@ mod tests {
         );
     }
 
+    #[test]
+    fn get_local_midnight() {
+        let deps = FakeDependencies::new();
+        let midnight = get_time(&mut deps.new_matcher_io(), true);
+
+        let midnight = DateTime::<Local>::from(midnight);
+        assert_eq!(midnight.time(), NaiveTime::from_hms_opt(0, 0, 0).unwrap());
+    }
+
     #[test]
     fn file_time_matcher_modified_changed_accessed() {
         let temp_dir = Builder::new()
@@ -724,11 +736,8 @@ mod tests {
                 // this file should already exist
                 let old_file = get_dir_entry_for("test_data", "simple");
                 let deps = FakeDependencies::new();
-                let matcher = NewerOptionMatcher::new(
-                    x_option.to_string(),
-                    y_option.to_string(),
-                    &old_file.path().to_string_lossy(),
-                );
+                let matcher =
+                    NewerOptionMatcher::new(x_option, y_option, &old_file.path().to_string_lossy());
 
                 assert!(
                     matcher
