@@ -3,7 +3,7 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-use super::{Matcher, MatcherIO, WalkEntry};
+use super::{ComparableValue, Matcher, MatcherIO, WalkEntry};
 
 #[cfg(unix)]
 use nix::unistd::User;
@@ -11,63 +11,39 @@ use nix::unistd::User;
 use std::os::unix::fs::MetadataExt;
 
 pub struct UserMatcher {
-    uid: Option<u32>,
+    uid: ComparableValue,
 }
 
 impl UserMatcher {
     #[cfg(unix)]
-    pub fn from_user_name(user: &str) -> UserMatcher {
+    pub fn from_user_name(user: &str) -> Option<Self> {
         // get uid from user name
-        let Ok(user) = User::from_name(user) else {
-            return UserMatcher { uid: None };
-        };
-
-        let Some(user) = user else {
-            // This if branch is to determine whether a certain user exists in the system.
-            // If a certain user does not exist in the system,
-            // the result will need to be returned according to
-            // the flag bit of whether to invert the result.
-            return UserMatcher { uid: None };
-        };
-
-        UserMatcher {
-            uid: Some(user.uid.as_raw()),
-        }
+        let user = User::from_name(user).ok()??;
+        let uid = user.uid.as_raw();
+        Some(Self::from_uid(uid))
     }
 
-    #[cfg(unix)]
-    pub fn from_uid(uid: u32) -> UserMatcher {
-        UserMatcher { uid: Some(uid) }
+    pub fn from_uid(uid: u32) -> Self {
+        Self::from_comparable(ComparableValue::EqualTo(uid as u64))
+    }
+
+    pub fn from_comparable(uid: ComparableValue) -> Self {
+        Self { uid }
     }
 
     #[cfg(windows)]
-    pub fn from_user_name(_user: &str) -> UserMatcher {
-        UserMatcher { uid: None }
-    }
-
-    #[cfg(windows)]
-    pub fn from_uid(_uid: u32) -> UserMatcher {
-        UserMatcher { uid: None }
-    }
-
-    pub fn uid(&self) -> &Option<u32> {
-        &self.uid
+    pub fn from_user_name(_user: &str) -> Option<Self> {
+        None
     }
 }
 
 impl Matcher for UserMatcher {
     #[cfg(unix)]
     fn matches(&self, file_info: &WalkEntry, _: &mut MatcherIO) -> bool {
-        let Ok(metadata) = file_info.metadata() else {
-            return false;
-        };
-
-        let file_uid = metadata.uid();
-
-        // When matching the -user parameter in find/matcher/mod.rs,
-        // it has been judged that the user does not exist and an error is returned.
-        // So use unwarp() directly here.
-        self.uid.unwrap() == file_uid
+        match file_info.metadata() {
+            Ok(metadata) => self.uid.matches(metadata.uid().into()),
+            Err(_) => false,
+        }
     }
 
     #[cfg(windows)]
@@ -134,7 +110,7 @@ mod tests {
             .unwrap()
             .name;
 
-        let matcher = UserMatcher::from_user_name(file_user.as_str());
+        let matcher = UserMatcher::from_user_name(file_user.as_str()).expect("user should exist");
         assert!(
             matcher.matches(&file_info, &mut matcher_io),
             "user should be the same"
@@ -144,14 +120,13 @@ mod tests {
         let time_string = Local::now().format("%Y%m%d%H%M%S").to_string();
         let matcher = UserMatcher::from_user_name(time_string.as_str());
         assert!(
-            matcher.uid().is_none(),
+            matcher.is_none(),
             "user {} should not be the same",
             time_string
         );
 
         // Testing user id
         let matcher = UserMatcher::from_uid(file_uid);
-        assert!(matcher.uid().is_some(), "user id {} should exist", file_uid);
         assert!(
             matcher.matches(&file_info, &mut matcher_io),
             "user id should match"
