@@ -10,6 +10,7 @@ use matchers::{Follow, WalkEntry};
 use std::cell::RefCell;
 use std::error::Error;
 use std::io::{stderr, stdout, Write};
+use std::path::PathBuf;
 use std::rc::Rc;
 use std::time::SystemTime;
 use walkdir::WalkDir;
@@ -25,6 +26,8 @@ pub struct Config {
     today_start: bool,
     no_leaf_dirs: bool,
     follow: Follow,
+    new_paths: Option<Vec<String>>,
+    files0_argument: Option<String>,
 }
 
 impl Default for Config {
@@ -43,6 +46,8 @@ impl Default for Config {
             // a compatibility item for GNU findutils.
             no_leaf_dirs: false,
             follow: Follow::Never,
+            new_paths: None, // This option exclusively for -files0-from argument.
+            files0_argument: None, //This option also is used for file0-from
         }
     }
 }
@@ -131,6 +136,16 @@ fn parse_args(args: &[&str]) -> Result<ParsedInfo, Box<dyn Error>> {
         paths.push(".".to_string());
     }
     let matcher = matchers::build_top_level_matcher(&args[i..], &mut config)?;
+    if let Some(new_paths) = &config.new_paths {
+        if paths.len() == 1 && paths[0] == "." {
+            paths = new_paths.to_vec();
+        } else {
+            return Err(From::from(format!(
+                "extra operand '{}'\nfile operands cannot be combined with -files0-from",
+                paths[0]
+            )));
+        }
+    }
     Ok(ParsedInfo {
         matcher,
         paths,
@@ -161,15 +176,26 @@ fn process_dir(
     // Slightly yucky loop handling here :-(. See docs for
     // WalkDirIterator::skip_current_dir for explanation.
     let mut it = walkdir.into_iter();
+    // As WalkDir seems not providing a function to check its stack,
+    // using current_dir is a workaround to check leaving directory.
+    let mut current_dir: Option<PathBuf> = None;
     while let Some(result) = it.next() {
         match WalkEntry::from_walkdir(result, config.follow) {
             Err(err) => {
                 ret = 1;
-                writeln!(&mut stderr(), "Error: {err}").unwrap()
+                writeln!(&mut stderr(), "Error: {err}").unwrap();
             }
             Ok(entry) => {
                 let mut matcher_io = matchers::MatcherIO::new(deps);
 
+                let new_dir = entry.path().parent().map(|x| x.to_path_buf());
+                if new_dir != current_dir {
+                    if let Some(dir) = current_dir.take() {
+                        matcher.finished_dir(dir.as_path(), &mut matcher_io);
+                    }
+                    current_dir = new_dir;
+                }
+
                 matcher.matches(&entry, &mut matcher_io);
                 match matcher_io.exit_code() {
                     0 => {}
@@ -186,6 +212,17 @@ fn process_dir(
         }
     }
 
+    let mut matcher_io = matchers::MatcherIO::new(deps);
+    if let Some(dir) = current_dir.take() {
+        matcher.finished_dir(dir.as_path(), &mut matcher_io);
+    }
+    matcher.finished(&mut matcher_io);
+    // This is implemented for exec +.
+    match matcher_io.exit_code() {
+        0 => {}
+        code => ret = code,
+    }
+
     ret
 }
 
@@ -236,6 +273,7 @@ Early alpha implementation. Currently the only expressions supported are
  -iname case-insensitive_filename_pattern
  -ilname case-insensitive_filename_pattern
  -regextype type
+ -files0-from
  -regex pattern
  -iregex pattern
  -type type_char
@@ -932,7 +970,7 @@ mod tests {
         #[cfg(not(target_os = "linux"))]
         let y_options = ["a", "B", "c", "m"];
 
-        for &x in x_options.iter() {
+        for &x in &x_options {
             for &y in &y_options {
                 let arg = &format!("-newer{x}{y}").to_string();
                 let deps = FakeDependencies::new();
@@ -1125,17 +1163,35 @@ mod tests {
         );
 
         // test uid
-        let deps = FakeDependencies::new();
-        let rc = find_main(
-            &[
-                "find",
-                "./test_data/simple/subdir",
-                "-uid",
-                &uid.to_string(),
-            ],
-            &deps,
-        );
-        assert_eq!(rc, 0);
+        for arg in ["-uid", "-user"] {
+            let deps = FakeDependencies::new();
+            let rc = find_main(
+                &["find", "./test_data/simple/subdir", arg, &uid.to_string()],
+                &deps,
+            );
+            assert_eq!(rc, 0);
+        }
+
+        // test -uid +N, -uid -N
+        if uid > 0 {
+            let deps = FakeDependencies::new();
+            let rc = find_main(
+                &[
+                    "find",
+                    "./test_data/simple/subdir",
+                    "-uid",
+                    &format!("+{}", uid - 1),
+                    "-uid",
+                    &format!("-{}", uid + 1),
+                ],
+                &deps,
+            );
+            assert_eq!(rc, 0);
+            assert_eq!(
+                deps.get_output_as_string(),
+                "./test_data/simple/subdir\n./test_data/simple/subdir/ABBBC\n"
+            );
+        }
 
         // test empty uid
         let deps = FakeDependencies::new();
@@ -1195,17 +1251,35 @@ mod tests {
         );
 
         // test gid
-        let deps = FakeDependencies::new();
-        let rc = find_main(
-            &[
-                "find",
-                "./test_data/simple/subdir",
-                "-gid",
-                gid.to_string().as_str(),
-            ],
-            &deps,
-        );
-        assert_eq!(rc, 0);
+        for arg in ["-gid", "-group"] {
+            let deps = FakeDependencies::new();
+            let rc = find_main(
+                &["find", "./test_data/simple/subdir", arg, &gid.to_string()],
+                &deps,
+            );
+            assert_eq!(rc, 0);
+        }
+
+        // test -gid +N, -gid -N
+        if gid > 0 {
+            let deps = FakeDependencies::new();
+            let rc = find_main(
+                &[
+                    "find",
+                    "./test_data/simple/subdir",
+                    "-gid",
+                    &format!("+{}", gid - 1),
+                    "-gid",
+                    &format!("-{}", gid + 1),
+                ],
+                &deps,
+            );
+            assert_eq!(rc, 0);
+            assert_eq!(
+                deps.get_output_as_string(),
+                "./test_data/simple/subdir\n./test_data/simple/subdir/ABBBC\n"
+            );
+        }
 
         // test empty gid
         let deps = FakeDependencies::new();
@@ -1338,20 +1412,24 @@ mod tests {
     }
 
     #[test]
-    fn find_fprint() {
-        let deps = FakeDependencies::new();
-        let rc = find_main(
-            &[
-                "find",
-                "./test_data/simple",
-                "-fprint",
-                "test_data/find_fprint",
-            ],
-            &deps,
-        );
-        assert_eq!(rc, 0);
+    fn find_fprinter() {
+        let printer = ["fprint", "fprint0"];
 
-        let _ = fs::remove_file("test_data/find_fprint");
+        for p in &printer {
+            let deps = FakeDependencies::new();
+            let rc = find_main(
+                &[
+                    "find",
+                    "./test_data/simple",
+                    format!("-{p}").as_str(),
+                    format!("test_data/find_{p}").as_str(),
+                ],
+                &deps,
+            );
+            assert_eq!(rc, 0);
+
+            let _ = fs::remove_file(format!("test_data/find_{p}"));
+        }
     }
 
     #[test]
@@ -1430,6 +1508,24 @@ mod tests {
         );
     }
 
+    #[test]
+    fn find_fprintf() {
+        let deps = FakeDependencies::new();
+        let rc = find_main(
+            &[
+                "find",
+                "./test_data/simple",
+                "-fprintf",
+                "test_data/find_fprintf",
+                "%h %H %p %P",
+            ],
+            &deps,
+        );
+        assert_eq!(rc, 0);
+
+        let _ = fs::remove_file("test_data/find_fprintf");
+    }
+
     #[test]
     #[cfg(unix)]
     fn test_ls() {
