@@ -10,6 +10,7 @@
 
 use assert_cmd::Command;
 use predicates::prelude::*;
+use regex::Regex;
 use serial_test::serial;
 use std::fs::{self, File};
 use std::io::{Read, Write};
@@ -73,6 +74,125 @@ fn two_matchers_one_matches() {
         .stdout(predicate::str::is_empty());
 }
 
+#[serial(working_dir)]
+#[test]
+fn files0_empty_file() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .args(["-files0-from", "./test_data/simple/abbbc"])
+        .assert()
+        .success()
+        .stderr(predicate::str::is_empty())
+        .stdout(predicate::str::is_empty());
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_file_basic_success() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .args(["-files0-from", "./test_data/simple/abbbc"])
+        .assert()
+        .success()
+        .stderr(predicate::str::is_empty())
+        .stdout(predicate::str::is_empty());
+
+    #[cfg(unix)]
+    {
+        let temp_dir = Builder::new().prefix("find_cmd_").tempdir().unwrap();
+        let test_file = temp_dir.path().join("test_files0");
+        let mut file = File::create(&test_file).expect("created test file");
+        file.write_all(b"./test_data/\0./test_data/simple/\0")
+            .expect("file write error");
+
+        Command::cargo_bin("find")
+            .expect("found binary")
+            .args(["-files0-from", &test_file.display().to_string()])
+            .assert()
+            .success()
+            .stderr(predicate::str::is_empty())
+            .stdout(predicate::str::contains("/test_data/"));
+    }
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_empty_pipe() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .args(["-files0-from", "-"])
+        .write_stdin(b"")
+        .assert()
+        .success()
+        .stderr(predicate::str::is_empty())
+        .stdout(predicate::str::is_empty());
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_pipe_basic() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .write_stdin(b"./test_data/simple\0./test_data/links")
+        .args(["-files0-from", "-"])
+        .assert()
+        .success()
+        .stderr(predicate::str::is_empty())
+        .stdout(predicate::str::contains("./test_data/"));
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_pipe_double_nul() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .write_stdin(b"./test_data/simple\0\0./test_data/links")
+        .args(["-files0-from", "-"])
+        .assert()
+        .success()
+        .stderr(predicate::str::contains("invalid zero-length file name"))
+        .stdout(predicate::str::contains("./test_data/"));
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_no_file() {
+    #[cfg(unix)]
+    {
+        Command::cargo_bin("find")
+            .expect("found binary")
+            .args(["-files0-from", "xyz.nonexistentFile"])
+            .assert()
+            .failure()
+            .stderr(predicate::str::contains("No such file or directory"))
+            .stdout(predicate::str::is_empty());
+    }
+    #[cfg(windows)]
+    {
+        Command::cargo_bin("find")
+            .expect("found binary")
+            .args(["-files0-from", "xyz.nonexistantFile"])
+            .assert()
+            .failure()
+            .stderr(predicate::str::contains(
+                "The system cannot find the file specified.",
+            ))
+            .stdout(predicate::str::is_empty());
+    }
+}
+
+#[serial(working_dir)]
+#[test]
+fn files0_basic() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .arg("-files0-from")
+        .assert()
+        .failure()
+        .stderr(predicate::str::contains("missing argument to -files0-from"))
+        .stdout(predicate::str::is_empty());
+}
+
 #[test]
 fn matcher_with_side_effects_at_end() {
     let temp_dir = Builder::new().prefix("find_cmd_").tempdir().unwrap();
@@ -367,6 +487,28 @@ fn find_printf() {
             ./test_data/simple/subdir/ABBBC subdir/ABBBC f\n",
         )));
 
+    fs::create_dir_all("a").expect("Failed to create directory 'a'");
+    let output = Command::cargo_bin("find")
+        .expect("found binary")
+        .args(["a", "-printf", "%A+"])
+        .assert()
+        .success()
+        .get_output()
+        .stdout
+        .clone();
+
+    let output_str = String::from_utf8(output).expect("Invalid UTF-8 in output");
+
+    println!("Actual output: '{}'", output_str.trim());
+
+    let re = Regex::new(r"^\d{4}-\d{2}-\d{2}\+\d{2}:\d{2}:\d{2}\.\d{9}0$")
+        .expect("Failed to compile regex");
+
+    assert!(
+        re.is_match(output_str.trim()),
+        "Output did not match expected timestamp format"
+    );
+
     Command::cargo_bin("find")
         .expect("found binary")
         .args([
@@ -952,24 +1094,32 @@ fn find_daystart() {
 
 #[test]
 #[serial(working_dir)]
-fn find_fprint() {
-    let _ = fs::remove_file("test_data/find_fprint");
+fn find_fprinter() {
+    let printer = ["fprint", "fprint0"];
 
-    Command::cargo_bin("find")
-        .expect("found binary")
-        .args(["test_data/simple", "-fprint", "test_data/find_fprint"])
-        .assert()
-        .success()
-        .stdout(predicate::str::is_empty())
-        .stderr(predicate::str::is_empty());
+    for p in &printer {
+        let _ = fs::remove_file(format!("test_data/find_{p}"));
 
-    // read test_data/find_fprint
-    let mut f = File::open("test_data/find_fprint").unwrap();
-    let mut contents = String::new();
-    f.read_to_string(&mut contents).unwrap();
-    assert!(contents.contains("test_data/simple"));
+        Command::cargo_bin("find")
+            .expect("found binary")
+            .args([
+                "test_data/simple",
+                format!("-{p}").as_str(),
+                format!("test_data/find_{p}").as_str(),
+            ])
+            .assert()
+            .success()
+            .stdout(predicate::str::is_empty())
+            .stderr(predicate::str::is_empty());
 
-    let _ = fs::remove_file("test_data/find_fprint");
+        // read test_data/find_fprint
+        let mut f = File::open(format!("test_data/find_{p}")).unwrap();
+        let mut contents = String::new();
+        f.read_to_string(&mut contents).unwrap();
+        assert!(contents.contains("test_data/simple"));
+
+        let _ = fs::remove_file(format!("test_data/find_{p}"));
+    }
 }
 
 #[test]
@@ -984,6 +1134,33 @@ fn find_follow() {
         .stderr(predicate::str::is_empty());
 }
 
+#[test]
+#[serial(working_dir)]
+fn find_fprintf() {
+    let _ = fs::remove_file("test_data/find_fprintf");
+
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .args([
+            "test_data/simple",
+            "-fprintf",
+            "test_data/find_fprintf",
+            "%h %H %p %P",
+        ])
+        .assert()
+        .success()
+        .stdout(predicate::str::is_empty())
+        .stderr(predicate::str::is_empty());
+
+    // read test_data/find_fprintf
+    let mut f = File::open("test_data/find_fprintf").unwrap();
+    let mut contents = String::new();
+    f.read_to_string(&mut contents).unwrap();
+    assert!(contents.contains("test_data/simple"));
+
+    let _ = fs::remove_file("test_data/find_fprintf");
+}
+
 #[test]
 #[serial(working_dir)]
 fn find_ls() {
@@ -994,3 +1171,14 @@ fn find_ls() {
         .success()
         .stderr(predicate::str::is_empty());
 }
+
+#[test]
+#[cfg(unix)]
+fn find_slashes() {
+    Command::cargo_bin("find")
+        .expect("found binary")
+        .args(["///", "-maxdepth", "0", "-name", "/"])
+        .assert()
+        .success()
+        .stderr(predicate::str::is_empty());
+}
